#!/usr/bin/env python3

'''Decrypt encrypted HTTP POST bodies to
http://stat.int.browser.baidu.com/stat_report.fcgi as sent by the global
edition of the Baidu Android browser.
'''

import sys

KEYS = (
    b'BAIDU_URL_BAV_INTERNATIONAL',
    b'SD_URL_BAIDU_INTERNATIONAL',
    b'BAIDU_URL_SECURE_INTERNATIONAL',
)

RANDTBL = (
    0x991539B1, 0x16A5BCE3, 0x6774A4CD, 0x3E01511E, 0x4E508AAA,
    0x61048C05, 0xF5500617, 0x846B7115, 0x6A19892C, 0x896A97AF,
    0xDB48F936, 0x14898454, 0x37FFD106, 0xB58BFF9C, 0x59E17104,
    0xCF918A49, 0x09378C83, 0x52C7A471, 0x8D293EA9, 0x1F4FC301,
    0xC3DB71BE, 0x39B44E1C, 0xF8A44EF9, 0x4C8B80B1, 0x19EDC328,
    0x87BF4BDD, 0xC9B240E5, 0xE9EE4B1B, 0x4382AEE7, 0x535B6B41,
    0xF3BEC5DA
)

FCRYPT_TABLE = (
    0xC1C0B6A1, 0xC7AEBAA2, 0xCF0A0DEF, 0xB1ADBDE6, 0xA5C83DB1,
    0xD9E90A0D, 0xDED6D3D7, 0xA0DB7CD, 0xCDB2B4BF, 0xBABDC9F2,
    0xDE9B1EC, 0xC1E3B20A, 0xC8A1BED6, 0xE90A0DA5, 0xD6D3D7D9,
    0xDB7CDDE, 0xCDB4BF0A, 0xBABDC9F2, 0xDE9B1EC, 0xC1E3B20A,
    0xC8A1BED6, 0xC20A0DBE, 0xB1ADBDFE, 0xDB8CDCC, 0xF4D9B00A,
    0xC1F9D5B4, 0xD30A0DF7, 0xB3F7BBA5, 0xDD5BFA4, 0xCFE3D30A,
    0xB5B3C7E8, 0xCD0A0DD7, 0xCBEFCEF2, 0xBEECCCAA, 0xD3D4D7BA,
    0xE20A0DC9, 0xC0C8C1EA, 0xCE0A0DAA, 0xC3D4B2CA, 0xB5F3B4A3,
    0xCB0A0DD8, 0xB3F7D6AD, 0xDA1B8C1, 0xC0AFD00A, 0xC2D9B0B4,
    0xD3F8D4C2, 0xD20A0DCE, 0xCEF9CDE4, 0xE1BFE1F4, 0xD4EACBC9,
    0xDEDB3C2, 0xCDA1C70A, 0xC9A7D1AC, 0xDEAC4D9, 0xBBE7B70A,
    0xC3FDD5AA, 0xCA0A0DAF, 0xD2FAC9E9, 0xDF8C6E2, 0xB3D3BB0A,
    0xE5BDB7E2, 0xD60A0DD9, 0xBDE3B5B8, 0xDBDC9AD, 0xD1A4BC0A,
    0xD7C4CEEF, 0xB70A0DD6, 0xB5C1CDE0, 0xCDEAC4B1, 0xBAA7BBF2,
    0xD40A0DEE, 0xB7C7BCF8, 0xB50A0DF1, 0xC1D0D6BD, 0xCBF7BBF7,
    0xC00A0DAE, 0xB7F4B6CB, 0xDDBD6C9, 0xBFE1BA0A, 0xCAF6B3D5,
    0xC30A0DC0, 0xC2A5C0A7, 0xD40A0DD8, 0xC8A1BEC4, 0xB4E4BCCB,
    0xDABC9BA, 0xC6C9B70A, 0xC1F1D3F0, 0xB0FDC8FA, 0xDF2CDD9,
    0xB5C1BD0A, 0xCCDCD6C3, 0xB3AEBAEC, 0xCF0A0DB9, 0xCFD5C8C4,
    0xDDAC8FB, 0xBAADBD0A, 0xD2E1BAD3, 0xC80A0DE7, 0xCEF2BBCB,
    0xB1E3D3AA, 0xC70A0DEE, 0xB9EFC7A7, 0xDEFD7A6, 0xC8ADCB0A,
    0xD3F8D4CB, 0xCBC0C6EB, 0xB60A0DB5, 0xCEF1BDF8, 0xC0BDCED2,
    0xDD8C2A5, 0xD2BBB20A, 0xB8E2D5AA, 0xB20A0DDF, 0xD5AAD2BB,
    0xD1E0B6E2, 0xB00A0DA9, 0xD2C3B5B2, 0xB3ECCCD0, 0xBDA6B1E9,
    0xB00A0DA3, 0xB2EAC8D1, 0xC8AACEC3, 0xDD8BDFD, 0xBDBBD20A,
    0xC5C5D2D8, 0xD20A0DB7, 0xD4D8BDBB, 0xDC0C3F9, 0xBDBBD20A,
    0xB6B9BBD8, 0xDFAB9AB, 0xC6ABCC0A, 0xBDC0CABD, 0xBB0A0DE7,
    0xCDF2C7B7, 0xC1CBB4AC, 0xC8C8B9, 0, 0,
    0, 0, 0, 0, 0,
    0, 0, 0, 0, 0
)

class FCryptRandom:
    def __init__(self):
        self.rand_type = 3
        self.rand_sep = 3
        self.fptr = 3
        self.rptr = 0
        self.rand_deg = 31
        self.tbl = list(RANDTBL)

def fcrypt_checksum_int(buf):
    cksum = 0
    for i in range(len(buf) // 2):
        cksum += buf[i * 2] | (buf[i * 2 + 1] << 8)
    if len(buf) % 2 == 1:
        cksum += buf[-1]
    return cksum

def fcrypt_random(rand):
    if rand.rand_type != 0:
        result = rand.tbl[rand.fptr] + rand.tbl[rand.rptr] & 0xffffffff
        rand.tbl[rand.fptr] = result
        rand.fptr = (rand.fptr + 1) % len(rand.tbl)
        rand.rptr = (rand.rptr + 1) % len(rand.tbl)
        result >>= 1
    else:
        result = ((2 * (1103515245 * rand.tbl[0] + 12345)) & 0xffffffff) >> 1
        rand.tbl[0] = result
    return result

def fcrypt_srandom(seed, rand):
    if not 0 <= rand.rand_type < 5:
        raise ValueError('illegal rand_type %d' % rand.rand_type)
    seed = 1 if seed == 0 else seed
    rand.tbl[0] = seed
    if rand.rand_type != 0:
        for i in range(1, rand.rand_deg):
            seed = 16807 * seed % 0x7fffffff
            rand.tbl[i] = seed
        rand.fptr = rand.rand_sep
        rand.rptr = 0
        for _ in range(10 * rand.rand_deg):
            fcrypt_random(rand)

def fcrypt_create(buf):
    cksum = fcrypt_checksum_int(buf)
    rand = FCryptRandom()
    fcrypt_srandom(cksum, rand)
    state = [fcrypt_random(rand) + f & 0xffffffff for f in FCRYPT_TABLE]
    return b''.join(i.to_bytes(4, 'little') for i in state)

def fcrypt_decode(fcrypt_state, buf):
    n = len(buf)
    idx = n & 0xff
    result = [None] * n
    if n >= 2:
        n -= 2
        crypt = buf[n] | (buf[n + 1] << 8)
        plain = crypt ^ fcrypt_state[2 * idx] ^ (fcrypt_state[2 * idx + 1] << 8)
        result[n] = plain & 0xff
        result[n + 1] = (plain >> 8) & 0xff
        idx = (idx + plain) & 0xff
    for i in range(n // 2):
        crypt = buf[2 * i] | (buf[2 * i + 1] << 8)
        plain = crypt ^ fcrypt_state[2 * idx] ^ (fcrypt_state[2 * idx + 1] << 8)
        result[i * 2] = plain & 0xff
        result[i * 2 + 1] = (plain >> 8) & 0xff
        idx = (idx + 1) & 0xff
    if n % 2 == 1:
        result[n - 1] = buf[n - 1] ^ fcrypt_state[2 * idx]
    return bytes(result)

def usage(code=1):
    sys.stderr.write('Usage: %s KEY_INDEX\n' % sys.argv[0])
    sys.exit(code)

def main():
    if len(sys.argv) != 2:
        usage()
    try:
        key_idx = int(sys.argv[1], 0)
    except ValueError:
        usage()
    if not 0 <= key_idx < 3:
        sys.stderr.write('Key index must be number n such tha 0 <= n < 3\n')
        sys.exit(1)
    crypt = sys.stdin.buffer.read()
    fcrypt_state = fcrypt_create(KEYS[key_idx])
    sys.stderr.write('%r\n' % (fcrypt_state,))
    plain = fcrypt_decode(fcrypt_state, crypt)
    sys.stdout.buffer.write(plain)

if __name__ == '__main__':
    main()
