#!/usr/bin/env python3

'''Decrypt many of Baidu's GET parameters included in many different requests
such as 'ua' and 'cuid'.
'''

MASK = 0x2D382324

ALPHABET = 'qogjOuCRNkfil5p4SQ3LAmxGKZTdesvB6z_YPahMI9t80rJyHW1DEwFbc7nUVX2-'
LOOKUP = {c: i for i, c in enumerate(ALPHABET)}

def ror4(x, n):
    return ((x >> n) | (x << (32 - n))) & 0xffffffff

def get_dword(buf, i):
    return buf[i] | (buf[i + 1] << 8) | (buf[i + 2] << 16) | (buf[i + 3] << 24)

def append_dword(lst, dword):
    lst.append(dword & 0xff)
    lst.append((dword >> 8) & 0xff)
    lst.append((dword >> 16) & 0xff)
    lst.append((dword >> 24) & 0xff)

def base64_decode(base64):
    base64 = ''.join(base64.split())
    remaining = ord(base64[-1]) - ord('A')
    base64 = base64[:-1]
    n = len(base64)
    if n % 4 != 0:
        raise ValueError('invalid base64 length %d not divisible by 4' % n)
    block_count = n // 4
    binary = []
    for i in range(block_count):
        block = base64[i * 4:i * 4 + 4]
        ns = [LOOKUP[c] for c in block]
        binary.append(ns[0] | ((ns[3] & 0x30) << 2))
        binary.append(ns[1] | ((ns[3] & 0x0c) << 4))
        binary.append(ns[2] | ((ns[3] & 0x03) << 6))
    length = block_count * 3
    if remaining:
        length -= 3 - remaining
    return bytes(binary), length

def main():
    import sys
    text = sys.stdin.read()
    decoded, length = base64_decode(text)
    n = len(decoded)
    decrypted = []
    for i4 in range(n // 4):
        i = i4 * 4
        dword = get_dword(decoded, i)
        dword ^= MASK
        dword = ror4(dword, 29)
        append_dword(decrypted, dword)
    sys.stdout.buffer.write(bytes(decrypted)[:length])

if __name__ == '__main__':
    main()
