#!/usr/bin/env python3

import sys
import zlib

'''Unparse binary format Baidu Browser Android uses to check for updates.'''

DATA = [(1, [(61448, [(32773, '326')], [(61448, [(16386, 1), (32771, 'j2')], [(39, [(16386, 1), (32773, '100732160')], []), (39, [(16386, 2), (32773, '立即升级浏览器6.1版本：<br>1、自定义壁纸，彰显个性；<br>2、截图加分享，一步到位；<br>3、趣星球能发视频弹幕啦！')], []), (39, [(16386, 3), (32773, 'http://d.update.baidu.com/lcmanage/index.php?r=DownloadAction&os=Android_Phone&prodline=baidubrowser&original_type=1&brand=unknown&model=sdk-phone-armv7&resolution=768%2A1184&imei=000000000000000&type=apk&channel=1000788h&sdk_version=22&version_name=6.0.15.0&ip=128.100.127.42&operator=0&auto=0&down_from=lc&original_id=6346&upgradeid=97&ori_version_name=6.1.13.0&ori_version_code=98&path=/original_new/4/1/1/1442283977841/baidubrowser.apk|20075264')], []), (39, [(16386, 6), (32773, '0')], [])])])])]

def unparse_leaves(leaves):
    leaves_bufs = []
    for type, obj in leaves:
        leaves_bufs.append(type.to_bytes(2, 'little'))
        if type < 32767:
            if type < 8191:
                intwidth = 1
            elif type < 16383:
                intwidth = 2
            else:
                intwidth = 4
            leaves_bufs.append(obj.to_bytes(intwidth, 'little'))
        else:
            utf8str = obj.encode('utf-8')
            leaves_bufs.append(len(utf8str).to_bytes(2, 'little'))
            leaves_bufs.append(utf8str)
    return b''.join(leaves_bufs)

def unparse_branch(branch):
    branch_bufs = []
    for type, leaves, branches in branch:
        branch_bufs.append(type.to_bytes(2, 'little'))
        leaves_buf = unparse_leaves(leaves)
        branch_bufs.append(len(leaves_buf).to_bytes(2, 'little'))
        branches_buf = b''
        if type < 61439 and branches:
            branches_buf = branches.encode('utf-16le')
        elif type < 65535:
            branches_buf = unparse_branch(branches)
        branch_bufs.append(len(branches_buf).to_bytes(4, 'little'))
        branch_bufs.append(leaves_buf)
        branch_bufs.append(branches_buf)
    return b''.join(branch_bufs)

def unparse_request(data):
    bufs = []
    n = len(data)
    entries = []
    for type, branch in data:
        buf = unparse_branch(branch)
        if type == 0 or 1:
            buf = len(buf).to_bytes(4, 'little') + zlib.compress(buf)
        entries.append((type, buf))
    start = 10 + 4 * n
    for i in range(n):
        type, buf = entries[i]
        x = ((type & 0xf) << 28) | (start & 0xfffffff)
        bufs.append(x.to_bytes(4, 'little'))
        start += len(buf)
    for _, buf in entries:
        bufs.append(buf)
    hdr = [b'db1\x00']
    hdr.append(start.to_bytes(4, 'little'))
    hdr.append(n.to_bytes(2, 'little'))
    return b''.join(hdr) + b''.join(bufs)

def main():
    buf = unparse_request(DATA)
    sys.stdout.buffer.write(buf)

if __name__ == '__main__':
    main()
